package com.uva.users.Controller;

import org.springframework.boot.configurationprocessor.json.JSONObject;
import org.springframework.web.bind.annotation.*;

import com.uva.users.Exception.BodegaException;
import com.uva.users.Exception.VinoException;
import com.uva.users.Model.Bodega;
import com.uva.users.Model.Vino;
import com.uva.users.Model.VinoConRelacion;
import com.uva.users.Repository.BodegaRepository;
import com.uva.users.Repository.VinoConRelacionRepository;
import com.uva.users.Repository.VinoRepository;

import jakarta.persistence.Column;

import java.util.*;

@RestController
@RequestMapping("/ejemploVinos")
public class ControllerVinos {
    private final VinoRepository repository;
    private final VinoConRelacionRepository repository2;
    private final BodegaRepository repository3;

    ControllerVinos(VinoRepository repository, VinoConRelacionRepository repository2, BodegaRepository repository3) {
        this.repository = repository;
        this.repository2 = repository2;
        this.repository3 = repository3;
    }

    @GetMapping()
    public List<Vino> getVinos() {
        return repository.findAll();
    }

    @PutMapping("/{id}")
    public String putVinos(@RequestBody Vino vino, @PathVariable("id") Integer identificador) {
        Optional<Vino> vinoExistente = repository.findById(identificador);

        if (vinoExistente.isPresent()) {
            Vino vinoActualizado = vinoExistente.get();
            vinoActualizado.setNombreComercial(vino.getNombreComercial());
            vinoActualizado.setBodegaId(vino.getBodegaId());
            vinoActualizado.setPrecio(vino.getPrecio());
            vinoActualizado.setDenominacion(vino.getDenominacion());
            vinoActualizado.setCategoria(vino.getCategoria());

            repository.save(vinoActualizado); // Guarda el vino actualizado
            return "Vino con ID " + identificador + " actualizado correctamente.";
        } else {
            return "Vino con ID " + identificador + " no encontrado.";
        }
    }

    @DeleteMapping("/{id}")
    public String deleteVino(@PathVariable("id") Integer identificador) {
        Optional<Vino> vinoExistente = repository.findById(identificador); // Buscar vino por ID

        if (vinoExistente.isPresent()) {
            repository.delete(vinoExistente.get()); // Eliminar el vino
            return "Vino con ID " + identificador + " eliminado correctamente.";
        } else {
            return "Vino con ID " + identificador + " no encontrado.";
        }
    }

    @PostMapping("/newVino")
    public String newVino(@RequestBody Vino newVino) {
        try {
            repository.save(newVino);
            return "Nuevo registro creado";
        } catch (Exception e) {
            throw new VinoException("Error al crear el nuevo registro.");
        }
    }





    // CODIGO NO IMPORTANTE PARA ESTA ENTREGA 

    @PostMapping("/vinos")
    public String postVinos(@RequestBody String body) {
        return "Post: " + body;
    }

    @DeleteMapping("/vinos")
    public String deleteVinos(@RequestBody String body) {
        return "Delete: " + body;
    }

    @GetMapping("/vinos/{id}/{tipo}")
    public String getVino(@PathVariable Long id, @PathVariable String tipo) {
        return "ID del vino: " + id + ", Tipo de vino: " + tipo;
    }

    @GetMapping("/cartaVinos")
    public String getCartaConQuery(@RequestParam String nombre) {
        return "Nombre del vino: " + nombre;
    }

    @GetMapping("/informacion/{id}")
    public String getVinoInfo(@PathVariable Long id,
            @RequestParam(required = false, defaultValue = "precio") List<String> campos) {
        String precio = "10.99";
        String denominacion = "Reserva";
        String bodega = "Abadia Retuerta";
        StringBuilder respuesta = new StringBuilder("ID del vino: " + id + ", Campos solicitados: ");

        for (String campo : campos) {
            switch (campo) {
                case "precio":
                    respuesta.append("precio: ").append(precio).append("; ");
                    break;
                case "denominacion":
                    respuesta.append("denominación: ").append(denominacion).append("; ");
                    break;
                case "bodega":
                    respuesta.append("bodega: ").append(bodega).append("; ");
                    break;
                default:
                    respuesta.append(campo).append(": no disponible; ");
            }
        }
        return respuesta.toString();
    }

    @GetMapping("/VinoPorNombre/{nombre}")
    public Vino getVinoPorNombre_Comercial(@PathVariable String nombre) {
        Vino vino = repository.findByNombreComercial(nombre)
                .orElseThrow(() -> new VinoException("no se ha encontrado el vino de nombre " + nombre));
        return vino;
    }

    @GetMapping("/VinoPorPrecio")
    public List<Vino> getVinoPorPrecio(@RequestParam Float precio1, @RequestParam Float precio2) {
        List<Vino> vinos = repository.findByPrecioBetween(precio1, precio2);
        return vinos;
    }

    @DeleteMapping("/BorrarPorDenominacion_Categoria")
    public List<Vino> deletePorDenominacion_Categoria(@RequestBody String json) {
        try {
            JSONObject jsonObjeto = new JSONObject(json);
            String denominacion = jsonObjeto.getString("denominacion");
            String categoria = jsonObjeto.getString("categoria");
            boolean existe = repository.existsVinoByDenominacionAndCategoria(denominacion, categoria);
            if (existe) {
                List<Vino> borrados = repository.deleteByDenominacionAndCategoria(denominacion, categoria);
                return borrados;
            } else {
                System.out.println("No existen vinos de la categoría y denominación");
            }
        } catch (Exception e) {
            System.out.println(e);
            return null;
        }
        return null;
    }

    @GetMapping("/VinoPorDenominacionOrdenadoPorNombre/{denominacion}")
    public List<Vino> getVinoPorDenominacionOrdenado(@PathVariable String denominacion) {
        List<Vino> vinos = repository.findByDenominacionOrdenadoNombreDesc(denominacion);
        return vinos;
    }

    @GetMapping("/VinoPorDenominacionYCategoria")
    public List<Vino> getVinoPorDenominacionYCategoria(@RequestParam String denominacion,
            @RequestParam String categoria) {
        List<Vino> vinos = repository.findByDenominacionYCategoria(denominacion, categoria);
        return vinos;
    }

    @PostMapping("/newBodega")
    public String newBodega(@RequestBody Bodega newBodega) {
        try {
            repository3.save(newBodega);
            return "Nuevo registro creado";
        } catch (Exception e) {
            throw new BodegaException("Error al crear el nuevo registro.");
        }
    }

    @PostMapping("/newVinoConRelacion")
    public String newVinoConRelacion(@RequestBody VinoConRelacion newVinoConRelacion) {
        try {
            repository2.save(newVinoConRelacion);
            return "Nuevo registro creado";
        } catch (Exception e) {
            throw new VinoException("Error al crear el nuevo registro.");
        }
    }

    @GetMapping(value = "/getBodega/{id}", produces = "application/json")
    public Optional<Bodega> getBodega(@PathVariable Integer id) {
        return repository3.findById(id);
    }

    @GetMapping(value = "/getVinoConRelacion/{id}", produces = "application/json")
    public Optional<VinoConRelacion> getVinoConRelacion(@PathVariable Integer id) {
        return repository2.findById(id);
    }
}
